(document:surround "/std/frame")
(document:insert "/std/functions")
(document:envelop with-translation _ "alterator-x11")

;;;;;;;;;;;;;;;;;;;;;; autodetect data
(define auto-detect (woo-read-first "/x11/autodetect"))
(define auto-xdriver (woo-get-option auto-detect 'xdriver "vesa"))
(define auto-monitor (woo-get-option auto-detect 'monitor "Monitor 1024x768"))
(define auto-xdriver-name (woo-get-option auto-detect 'xdrivername))
(define auto-card-name (woo-get-option auto-detect 'cardname #f))
(define auto-xdepth (woo-get-option auto-detect 'xdepth #f))

;;;;;;;;;;;;;;;;;;;;;; current data
(define current-data (woo-read-first "/x11"))
(define current-xdriver (woo-get-option current-data 'xdriver auto-xdriver))
(define current-xdriver-name (woo-get-option current-data 'xdrivername auto-xdriver-name))
(define current-monitor (woo-get-option current-data 'monitor auto-monitor))
(define current-xdepth (woo-get-option current-data 'xdepth #f))
(define current-xresolution (woo-get-option current-data 'xresolution #f))
(define current-runlevel (woo-get-option (woo-read-first "/inittab") 'runlevel 5))

;;;;;;;;;;;;;;;;;;;;;;
(define module (_ "Display properties"))

(define autodetect (woo-read-first "/monitors/auto_detect"))

(define recommended_resolution (woo-get-option autodetect 'recommended))
(define ddcresolutions
  (if (empty-string? (woo-get-option autodetect 'detected))
      '()
      (string-cut-repeated  (woo-get-option autodetect 'detected) #\space)))

(define selected-driver (make-cell (cons current-xdriver current-xdriver-name)))
(define selected-monitor (make-cell current-monitor))

;;return selected driver or autodetected data
(define (get-xdriver)
  (or (car (cell-ref selected-driver)) auto-xdriver))

;;return selected monitor or autodetected data
(define (get-monitor)
  (or (cell-ref selected-monitor) auto-monitor))

;;color names
(define xdepth-list  (list (cons (_ "256 colors (8 bit)") "8")
                      (cons (_ "32 thousand colors (15 bits)") "15")
                      (cons (_ "65 thousand colors (16 bits)") "16")
                      (cons (_ "16 million colors (24 bits)") "24")
                      (cons (_ "4 billion colors (32 bits)") "32")))

(define all-resolution-list '("2048x1536"
                              "1920x1200"
                              "1920x1080"
                              "1680x1050"
                              "1600x1200"
                              "1600x1024"
                              "1600x900"
                              "1440x900"
                              "1400x1050"
                              "1280x1024"
                              "1280x960"
                              "1280x854"
                              "1280x800"
                              "1280x768"
                              "1280x720"
                              "1152x870"
                              "1152x864"
                              "1120x840"
                              "1024x768"
                              "1024x736"
                              "1024x600"
                              "960x720"
                              "848x480"
                              "832x624"
                              "800x600"
                              "720x576"
                              "640x480"))

;;return ddcresultions if it detected
(define xresolution-list (cons "auto" (if (null? ddcresolutions)
                                           all-resolution-list
                                           ddcresolutions)))

(define (get-xdepth)
  (cdr (list-ref xdepth-list (depth current))))

(define (recommended-xdepth driver)
  (woo-get-option (woo-read-first "/x11/xdriver" 'name driver)
                  'xdepth
                  #f))

(define (default-xdepth)
  (let* ((xdepth (or current-xdepth auto-xdepth))
         (recommended-xdepth (recommended-xdepth (or current-xdriver recommended-xdriver))))
         (list
          rows (map (lambda(x)
                      (if (and (string? recommended-xdepth)
		               (string=? (cdr x) recommended-xdepth))
                          (string-append (car x) (_ " - recommended"))
                          (car x)))
                    xdepth-list)
          current (or (list-index (lambda(x) (string=? (cdr x) xdepth))
                                  xdepth-list)
                      2))))

(define (default-xresolution)
  (list
   rows  (map (lambda(res) (if (string=? res recommended_resolution)
                               (string-append res (_ " - recommended"))
                               res))
              xresolution-list)
   current (string-list-index (or current-xresolution
                                  recommended_resolution
                                  "auto")
                              xresolution-list)))

(define (get-xresolution)
  (if (string=? (resolution text) "auto")
      "auto"
      (string-join (reverse (member (list-ref xresolution-list (resolution current))
                                    all-resolution-list))
                   ",")))

;; write data to temporary X11 config file
(define (apply-data)
  (woo-write "/x11" 'xdriver (get-xdriver)
                    'monitor (get-monitor)
		    'xdepth (get-xdepth)
                    'xresolution (get-xresolution))
  (woo-write "/inittab"
             'runlevel (if (startx11 state) 5 3)))

;;made temporary X11 config file, fill with data and commit
(define (apply-config)
  (woo-catch/message
   (thunk
     (apply-data)
     (woo-write "/x11/commit"))))

(define (apply-autoinstall)
  (woo-write "/autoinstall/x11/"
             'xdriver (get-xdriver)
             'monitor (get-monitor)
	     'xdepth (get-xdepth)
             'xresolution (get-xresolution))
  (woo-write "/autoinstall/inittab"
             'runlevel (if (startx11 state) 5 3)))

;;made temporary X11 config file, fill with data and run test
(define (test-not-apply)
  (woo-catch/message
   (thunk (apply-data)
          (woo-write "/x11/test"))))

;; save data if url changed
(define (change url)
  (woo-write "/x11" 'xdriver (get-xdriver)
                    'monitor (get-monitor)
		    'xdepth (get-xdepth)
                    'xresolution (get-xresolution))
  (frame:replace url))

;;;;;;;;;;;;;;;;;;;;;;;;; ui

(gridbox columns "20;15;35;20"
	           ;;0
		   (spacer)
		   (label (_ "Detected hardware:"))
		   (spacer)
		   (spacer)
	           ;;1	   
                   (spacer) (label (bold (_ "Video card"))) (label auto-card-name) (spacer)
                   (spacer) (label (bold (_ "Monitor"))) (label auto-monitor) (spacer)
		   (spacer) (separator) (separator) (spacer)
		   ;;2
		   (spacer)
		   (label (_ "Current settings:"))
		   (spacer)
		   (spacer)
                   ;;3
                   (spacer)
                   (document:id ch-drv (button (_ "Driver:...")))
                   (document:id drv (label (cdr (cell-ref selected-driver))))
                   (spacer)
                   ;;4
                   (spacer)
                   (document:id ch-mon (button (_ "Monitor:...")))
                   (document:id mon (label (get-monitor)))
                   (spacer)
                   ;;5
                   (spacer)
                   (label (bold (_ "Color depth")))
                   (document:id depth (combobox (default-xdepth)))
                   (spacer)
                   ;;6
                   (spacer)
                   (label (bold (_ "Resolution")))
                   (document:id resolution (combobox (default-xresolution)))
                   (spacer)
		   ;;7
		   (spacer)
		   (separator)
		   (spacer)
		   (spacer)
		   ;;8
		   (spacer)
		   (document:id startx11 (checkbox (_ "Start X11 on system booting")))
		   (spacer)
		   (spacer))

;;;;;;;;;;;;;;;;;;; buttons

(hbox
 align "center"
 (or (global 'frame:next-button)
     (button (_ "Exit") (when clicked (document:end))))
 (button (_ "Test") (when clicked (test-not-apply)))
 (or (global 'frame:next-button)
     (button (_ "Save") (when clicked (apply-config)))))

;;;;;;;;;;;;;;;;;;; tooltips
(define (tt . args)
  (plist-for-each (lambda(x y) (x tooltip y)) args))

(tt startx11 (_ "Computer will start into graphical login if checked")
    resolution (_ "Monitor resolution, select auto if you want to use auto-detect")
    depth (_ "Select default depth for videocard")
    ch-mon (_ "Click for select your monitor vendor and model")
    ch-drv (_ "Change current driver for videocard"))

;;;;;;;;;;;;;;;;;;; actions
(ch-mon (when clicked (change "/x11/monitors")))
(ch-drv (when clicked (change "/x11/cards")))

(startx11 state (or (= current-runlevel 5) (->bool (global 'frame:next-button))))

;;;;;;;;;;;;;;;;;;; installer specific
(frame:on-next (thunk
                (or (and (apply-config) (apply-autoinstall))
                    'cancel)))
